@@ -18,7 +18,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 // Qt
-#include <QtTest/QtTest>
+#include <QtTest>
 // client
 #include "../../src/client/connection_thread.h"
 #include "../../src/client/event_queue.h"
@@ -28,6 +28,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 // server
 #include "../../src/server/display.h"
 #include "../../src/server/plasmawindowmanagement_interface.h"
+#include "../../src/server/plasmavirtualdesktop_interface.h"
 
 #include <linux/input.h>
 
@@ -45,6 +46,8 @@ typedef void (KWayland::Server::PlasmaWindowInterface::*ServerWindowQuint32Sette
 Q_DECLARE_METATYPE(ServerWindowQuint32Setter)
 typedef void (KWayland::Server::PlasmaWindowInterface::*ServerWindowVoidSetter)();
 Q_DECLARE_METATYPE(ServerWindowVoidSetter)
+typedef void (KWayland::Server::PlasmaWindowInterface::*ServerWindowIconSetter)(const QIcon&);
+Q_DECLARE_METATYPE(ServerWindowIconSetter)
 
 class PlasmaWindowModelTest : public QObject
 {
@@ -67,7 +70,6 @@ private Q_SLOTS:
     void testIsMinimized();
     void testIsKeepAbove();
     void testIsKeepBelow();
-    void testIsOnAllDesktops();
     void testIsDemandingAttention();
     void testSkipTaskbar();
     void testSkipSwitcher();
@@ -81,7 +83,7 @@ private Q_SLOTS:
     void testTitle();
     void testAppId();
     void testPid();
-    void testVirtualDesktop();
+    void testVirtualDesktops();
     // TODO icon: can we ensure a theme is installed on CI?
     void testRequests();
     // TODO: minimized geometry
@@ -96,6 +98,7 @@ private:
     Display *m_display = nullptr;
     PlasmaWindowManagementInterface *m_pwInterface = nullptr;
     PlasmaWindowManagement *m_pw = nullptr;
+    KWayland::Server::PlasmaVirtualDesktopManagementInterface *m_plasmaVirtualDesktopManagementInterface = nullptr;
     ConnectionThread *m_connection = nullptr;
     QThread *m_thread = nullptr;
     EventQueue *m_queue = nullptr;
@@ -113,6 +116,12 @@ void PlasmaWindowModelTest::init()
     m_display->createShm();
     m_pwInterface = m_display->createPlasmaWindowManagement();
     m_pwInterface->create();
+    m_plasmaVirtualDesktopManagementInterface = m_display->createPlasmaVirtualDesktopManagement(m_display);
+    m_plasmaVirtualDesktopManagementInterface->create();
+    QVERIFY(m_plasmaVirtualDesktopManagementInterface->isValid());
+    m_plasmaVirtualDesktopManagementInterface->createDesktop("desktop1");
+    m_plasmaVirtualDesktopManagementInterface->createDesktop("desktop2");
+    m_pwInterface->setPlasmaVirtualDesktopManagementInterface(m_plasmaVirtualDesktopManagementInterface);
 
     // setup connection
     m_connection = new KWayland::Client::ConnectionThread;
@@ -153,6 +162,7 @@ void PlasmaWindowModelTest::cleanup()
         variable = nullptr; \
     }
     CLEANUP(m_pw)
+    CLEANUP(m_plasmaVirtualDesktopManagementInterface)
     CLEANUP(m_queue)
     if (m_connection) {
         m_connection->deleteLater();
@@ -326,7 +336,7 @@ void PlasmaWindowModelTest::testDefaultData_data()
     QTest::newRow("IsKeepAbove")          << int(PlasmaWindowModel::IsKeepAbove) << QVariant(false);
     QTest::newRow("IsKeepBelow")          << int(PlasmaWindowModel::IsKeepBelow) << QVariant(false);
     QTest::newRow("VirtualDesktop")       << int(PlasmaWindowModel::VirtualDesktop) << QVariant(0);
-    QTest::newRow("IsOnAllDesktops")      << int(PlasmaWindowModel::IsOnAllDesktops) << QVariant(false);
+    QTest::newRow("IsOnAllDesktops")      << int(PlasmaWindowModel::IsOnAllDesktops) << QVariant(true);
     QTest::newRow("IsDemandingAttention") << int(PlasmaWindowModel::IsDemandingAttention) << QVariant(false);
     QTest::newRow("IsShadeable")           << int(PlasmaWindowModel::IsShadeable) << QVariant(false);
     QTest::newRow("IsShaded")             << int(PlasmaWindowModel::IsShaded) << QVariant(false);
@@ -401,11 +411,6 @@ void PlasmaWindowModelTest::testIsKeepBelow()
     QVERIFY(testBooleanData(PlasmaWindowModel::IsKeepBelow, &PlasmaWindowInterface::setKeepBelow));
 }
 
-void PlasmaWindowModelTest::testIsOnAllDesktops()
-{
-    QVERIFY(testBooleanData(PlasmaWindowModel::IsOnAllDesktops, &PlasmaWindowInterface::setOnAllDesktops));
-}
-
 void PlasmaWindowModelTest::testIsDemandingAttention()
 {
     QVERIFY(testBooleanData(PlasmaWindowModel::IsDemandingAttention, &PlasmaWindowInterface::setDemandsAttention));
@@ -549,7 +554,7 @@ void PlasmaWindowModelTest::testPid()
     QCOMPARE(model->data(index, PlasmaWindowModel::Pid).toInt(), 1337);
 }
 
-void PlasmaWindowModelTest::testVirtualDesktop()
+void PlasmaWindowModelTest::testVirtualDesktops()
 {
     auto model = m_pw->createWindowModel();
     QVERIFY(model);
@@ -564,17 +569,36 @@ void PlasmaWindowModelTest::testVirtualDesktop()
     QVERIFY(dataChangedSpy.isValid());
 
     const QModelIndex index = model->index(0);
-    QCOMPARE(model->data(index, PlasmaWindowModel::VirtualDesktop).toInt(), 0);
+    QCOMPARE(model->data(index, PlasmaWindowModel::VirtualDesktops).toStringList(), QStringList());
 
-    w->setVirtualDesktop(1);
+    w->addPlasmaVirtualDesktop("desktop1");
+    QVERIFY(dataChangedSpy.wait());
+    QCOMPARE(dataChangedSpy.count(), 2);
+    QCOMPARE(dataChangedSpy.first().first().toModelIndex(), index);
+    QCOMPARE(dataChangedSpy.last().first().toModelIndex(), index);
+
+    QCOMPARE(dataChangedSpy.first().last().value<QVector<int>>(), QVector<int>{int(PlasmaWindowModel::VirtualDesktops)});
+    QCOMPARE(dataChangedSpy.last().last().value<QVector<int>>(), QVector<int>{int(PlasmaWindowModel::IsOnAllDesktops)});
+
+    QCOMPARE(model->data(index, PlasmaWindowModel::VirtualDesktops).toStringList(), QStringList({"desktop1"}));
+    QCOMPARE(model->data(index, PlasmaWindowModel::IsOnAllDesktops).toBool(), false);
+
+    dataChangedSpy.clear();
+    w->addPlasmaVirtualDesktop("desktop2");
     QVERIFY(dataChangedSpy.wait());
     QCOMPARE(dataChangedSpy.count(), 1);
     QCOMPARE(dataChangedSpy.last().first().toModelIndex(), index);
-    QCOMPARE(dataChangedSpy.last().last().value<QVector<int>>(), QVector<int>{int(PlasmaWindowModel::VirtualDesktop)});
-    QCOMPARE(model->data(index, PlasmaWindowModel::VirtualDesktop).toInt(), 1);
+    QCOMPARE(dataChangedSpy.last().last().value<QVector<int>>(), QVector<int>{int(PlasmaWindowModel::VirtualDesktops)});
+    QCOMPARE(model->data(index, PlasmaWindowModel::VirtualDesktops).toStringList(), QStringList({"desktop1", "desktop2"}));
+    QCOMPARE(model->data(index, PlasmaWindowModel::IsOnAllDesktops).toBool(), false);
+
+    w->removePlasmaVirtualDesktop("desktop2");
+    w->removePlasmaVirtualDesktop("desktop1");
+    QVERIFY(dataChangedSpy.wait());
+    QCOMPARE(dataChangedSpy.last().last().value<QVector<int>>(), QVector<int>{int(PlasmaWindowModel::IsOnAllDesktops)});
+    QCOMPARE(model->data(index, PlasmaWindowModel::VirtualDesktops).toStringList(), QStringList({}));
+    QCOMPARE(model->data(index, PlasmaWindowModel::IsOnAllDesktops).toBool(), true);
 
-    // setting to same should not trigger
-    w->setVirtualDesktop(1);
     QVERIFY(!dataChangedSpy.wait(100));
 }
 
@@ -839,29 +863,32 @@ void PlasmaWindowModelTest::testChangeWindowAfterModelDestroy_data()
     QTest::addColumn<QVariant>("setter");
     QTest::addColumn<QVariant>("value");
 
-    QTest::newRow("active")           << &PlasmaWindow::activeChanged                   << qVariantFromValue(&PlasmaWindowInterface::setActive)                   << QVariant(true);
-    QTest::newRow("minimized")        << &PlasmaWindow::minimizedChanged                << qVariantFromValue(&PlasmaWindowInterface::setMinimized)                << QVariant(true);
-    QTest::newRow("fullscreen")       << &PlasmaWindow::fullscreenChanged               << qVariantFromValue(&PlasmaWindowInterface::setFullscreen)               << QVariant(true);
-    QTest::newRow("keepAbove")        << &PlasmaWindow::keepAboveChanged                << qVariantFromValue(&PlasmaWindowInterface::setKeepAbove)                << QVariant(true);
-    QTest::newRow("keepBelow")        << &PlasmaWindow::keepBelowChanged                << qVariantFromValue(&PlasmaWindowInterface::setKeepBelow)                << QVariant(true);
-    QTest::newRow("maximized")        << &PlasmaWindow::maximizedChanged                << qVariantFromValue(&PlasmaWindowInterface::setMaximized)                << QVariant(true);
-    QTest::newRow("demandsAttention") << &PlasmaWindow::demandsAttentionChanged         << qVariantFromValue(&PlasmaWindowInterface::setDemandsAttention)         << QVariant(true);
-    QTest::newRow("closeable")        << &PlasmaWindow::closeableChanged                << qVariantFromValue(&PlasmaWindowInterface::setCloseable)                << QVariant(true);
-    QTest::newRow("minimizeable")     << &PlasmaWindow::minimizeableChanged             << qVariantFromValue(&PlasmaWindowInterface::setMinimizeable)             << QVariant(true);
-    QTest::newRow("maximizeable")     << &PlasmaWindow::maximizeableChanged             << qVariantFromValue(&PlasmaWindowInterface::setMaximizeable)             << QVariant(true);
-    QTest::newRow("fullscreenable")   << &PlasmaWindow::fullscreenableChanged           << qVariantFromValue(&PlasmaWindowInterface::setFullscreenable)           << QVariant(true);
-    QTest::newRow("skipTaskbar")      << &PlasmaWindow::skipTaskbarChanged              << qVariantFromValue(&PlasmaWindowInterface::setSkipTaskbar)              << QVariant(true);
-    QTest::newRow("shadeable")        << &PlasmaWindow::shadeableChanged                << qVariantFromValue(&PlasmaWindowInterface::setShadeable)                << QVariant(true);
-    QTest::newRow("shaded")           << &PlasmaWindow::shadedChanged                   << qVariantFromValue(&PlasmaWindowInterface::setShaded)                   << QVariant(true);
-    QTest::newRow("movable")          << &PlasmaWindow::movableChanged                  << qVariantFromValue(&PlasmaWindowInterface::setMovable)                  << QVariant(true);
-    QTest::newRow("resizable")        << &PlasmaWindow::resizableChanged                << qVariantFromValue(&PlasmaWindowInterface::setResizable)                << QVariant(true);
-    QTest::newRow("vdChangeable")     << &PlasmaWindow::virtualDesktopChangeableChanged << qVariantFromValue(&PlasmaWindowInterface::setVirtualDesktopChangeable) << QVariant(true);
-    QTest::newRow("onallDesktop")     << &PlasmaWindow::onAllDesktopsChanged            << qVariantFromValue(&PlasmaWindowInterface::setOnAllDesktops)            << QVariant(true);
-    QTest::newRow("title")            << &PlasmaWindow::titleChanged                    << qVariantFromValue(&PlasmaWindowInterface::setTitle)                    << QVariant(QStringLiteral("foo"));
-    QTest::newRow("appId")            << &PlasmaWindow::appIdChanged                    << qVariantFromValue(&PlasmaWindowInterface::setAppId)                    << QVariant(QStringLiteral("foo"));
-    QTest::newRow("icon" )            << &PlasmaWindow::iconChanged                     << qVariantFromValue(&PlasmaWindowInterface::setThemedIconName)           << QVariant(QStringLiteral("foo"));
-    QTest::newRow("vd")               << &PlasmaWindow::virtualDesktopChanged           << qVariantFromValue(&PlasmaWindowInterface::setVirtualDesktop)           << QVariant(2u);
-    QTest::newRow("unmapped")         << &PlasmaWindow::unmapped                        << qVariantFromValue(&PlasmaWindowInterface::unmap)                       << QVariant();
+    QTest::newRow("active")           << &PlasmaWindow::activeChanged                   << QVariant::fromValue(&PlasmaWindowInterface::setActive)                   << QVariant(true);
+    QTest::newRow("minimized")        << &PlasmaWindow::minimizedChanged                << QVariant::fromValue(&PlasmaWindowInterface::setMinimized)                << QVariant(true);
+    QTest::newRow("fullscreen")       << &PlasmaWindow::fullscreenChanged               << QVariant::fromValue(&PlasmaWindowInterface::setFullscreen)               << QVariant(true);
+    QTest::newRow("keepAbove")        << &PlasmaWindow::keepAboveChanged                << QVariant::fromValue(&PlasmaWindowInterface::setKeepAbove)                << QVariant(true);
+    QTest::newRow("keepBelow")        << &PlasmaWindow::keepBelowChanged                << QVariant::fromValue(&PlasmaWindowInterface::setKeepBelow)                << QVariant(true);
+    QTest::newRow("maximized")        << &PlasmaWindow::maximizedChanged                << QVariant::fromValue(&PlasmaWindowInterface::setMaximized)                << QVariant(true);
+    QTest::newRow("demandsAttention") << &PlasmaWindow::demandsAttentionChanged         << QVariant::fromValue(&PlasmaWindowInterface::setDemandsAttention)         << QVariant(true);
+    QTest::newRow("closeable")        << &PlasmaWindow::closeableChanged                << QVariant::fromValue(&PlasmaWindowInterface::setCloseable)                << QVariant(true);
+    QTest::newRow("minimizeable")     << &PlasmaWindow::minimizeableChanged             << QVariant::fromValue(&PlasmaWindowInterface::setMinimizeable)             << QVariant(true);
+    QTest::newRow("maximizeable")     << &PlasmaWindow::maximizeableChanged             << QVariant::fromValue(&PlasmaWindowInterface::setMaximizeable)             << QVariant(true);
+    QTest::newRow("fullscreenable")   << &PlasmaWindow::fullscreenableChanged           << QVariant::fromValue(&PlasmaWindowInterface::setFullscreenable)           << QVariant(true);
+    QTest::newRow("skipTaskbar")      << &PlasmaWindow::skipTaskbarChanged              << QVariant::fromValue(&PlasmaWindowInterface::setSkipTaskbar)              << QVariant(true);
+    QTest::newRow("shadeable")        << &PlasmaWindow::shadeableChanged                << QVariant::fromValue(&PlasmaWindowInterface::setShadeable)                << QVariant(true);
+    QTest::newRow("shaded")           << &PlasmaWindow::shadedChanged                   << QVariant::fromValue(&PlasmaWindowInterface::setShaded)                   << QVariant(true);
+    QTest::newRow("movable")          << &PlasmaWindow::movableChanged                  << QVariant::fromValue(&PlasmaWindowInterface::setMovable)                  << QVariant(true);
+    QTest::newRow("resizable")        << &PlasmaWindow::resizableChanged                << QVariant::fromValue(&PlasmaWindowInterface::setResizable)                << QVariant(true);
+    QTest::newRow("vdChangeable")     << &PlasmaWindow::virtualDesktopChangeableChanged << QVariant::fromValue(&PlasmaWindowInterface::setVirtualDesktopChangeable) << QVariant(true);
+    QTest::newRow("onallDesktop")     << &PlasmaWindow::onAllDesktopsChanged            << QVariant::fromValue(&PlasmaWindowInterface::setOnAllDesktops)            << QVariant(true);
+    QTest::newRow("title")            << &PlasmaWindow::titleChanged                    << QVariant::fromValue(&PlasmaWindowInterface::setTitle)                    << QVariant(QStringLiteral("foo"));
+    QTest::newRow("appId")            << &PlasmaWindow::appIdChanged                    << QVariant::fromValue(&PlasmaWindowInterface::setAppId)                    << QVariant(QStringLiteral("foo"));
+#if KWAYLANDSERVER_ENABLE_DEPRECATED_SINCE(5, 28)
+    QTest::newRow("iconname" )            << &PlasmaWindow::iconChanged                     << QVariant::fromValue(&PlasmaWindowInterface::setThemedIconName)           << QVariant(QStringLiteral("foo"));
+#endif
+    QTest::newRow("icon" )            << &PlasmaWindow::iconChanged                     << QVariant::fromValue(&PlasmaWindowInterface::setIcon)                     << QVariant::fromValue(QIcon::fromTheme(QStringLiteral("foo")));
+    QTest::newRow("vd")               << &PlasmaWindow::virtualDesktopChanged           << QVariant::fromValue(&PlasmaWindowInterface::setVirtualDesktop)           << QVariant(2u);
+    QTest::newRow("unmapped")         << &PlasmaWindow::unmapped                        << QVariant::fromValue(&PlasmaWindowInterface::unmap)                       << QVariant();
 }
 
 void PlasmaWindowModelTest::testChangeWindowAfterModelDestroy()
